@@ -12,6 +12,8 @@ from kubernetes import client, config
 from requests.packages.urllib3.util.retry import Retry
 from urllib3.exceptions import MaxRetryError
 from google.cloud import pubsub_v1
+import jwt
+import time
 
 logging.basicConfig(level=logging.INFO)
 
@@ -48,6 +50,84 @@ class InternalError(Exception):
     def __str__(self):
         return json.dumps({"source": self.source, "message": self.message})
 
+class TimeRange:
+
+    def __new__(self, timeRangeType):
+
+        class AbsoluteTimeRange:
+
+            def __init__(self):
+                pass
+
+            startTime = None
+            endTime = None
+
+            def setStartTime(self, startTime: int):
+                self.startTime = startTime
+                return self
+
+            def setEndTime(self, endTime: int):
+                self.endTime = endTime
+                return self
+
+            def getStartTime(self):
+                return self.startTime
+
+            def getEndTime(self):
+                return self.endTime
+
+            def getTimeRangeDict(self):
+                timeDict = dict()
+                timeDict["type"] = timeRangeType
+                timeDict["startTime"]= self.startTime
+                timeDict["endTime"] = self.endTime
+                return timeDict
+
+            def __str__(self) -> str:
+                return (f"Range Type : {timeRangeType},\nStart Time : {self.startTime},\nEnd Time : {self.endTime}")
+
+        class RelativeTimeRange:
+
+            unit = None
+            amount = None
+
+            def __init__(self):
+                pass
+
+            def setAmount(self, amount: int):
+                self.amount = amount
+                return self
+
+            def setUnit(self, unit: str):
+                self.unit = unit
+                return self
+
+            def getAmount(self):
+                return self.amount
+
+            def getUnit(self):
+                return self.unit
+
+            def getTimeRangeDict(self):
+                timeDict = dict()
+                timeDict["type"] = timeRangeType
+                timeDict["amount"]= self.amount
+                timeDict["unit"] = self.unit
+                return timeDict
+
+            def __str__(self) -> str:
+                return (f"Range Type : {timeRangeType},\nAmount : {self.amount},\nUnit : {self.unit}")
+
+            # return RelativeTimeRange()
+
+        if timeRangeType=='absolute':
+            return AbsoluteTimeRange()
+        elif timeRangeType=='relative':
+            return RelativeTimeRange()
+
+    def getTimeRangeDict(self):
+        pass
+
 def get_client_secret():
     if os.getenv("KUBERNETES_SERVICE_HOST"):
         config.load_incluster_config()
@@ -175,7 +255,53 @@ def datetime_handler(val):
         return val.isoformat()
     return str(val)
 
-def forward_logs(log_data,type="findings"):    
+def create_snapshot(ingestion_type,metadata):
+    headers = {
+        "x-dassana-app-id":f"{app_id}",
+        "x-dassana-token":f"{dassana_token}",
+        "x-dassana-ingestion-type":f"{ingestion_type}"
+    }
+    
+    resp = requests.post(f"https://ingestion.dassana.dev/snapshot", headers=headers,json=metadata)
+    snapshot = resp.json()
+
+    currentTs = round(time.time() * 1000)
+    timeRange = currentTs - 3600000
+    if snapshot['status'] == "IN_PROGRESS":
+        if timeRange <= snapshot['lastUpdatedTs'] <= currentTs:
+            logging.info("wait for previous snapshot to end")
+            return None
+        else:
+            snapshot_id = snapshot.get('id')
+            end_snapshot(ingestion_type,snapshot_id,'cancel',False)
+            result = create_snapshot(ingestion_type,metadata)
+            return result
+    else:
+        return snapshot
+    
+def update_snapshot(ingestion_type,snapshot_id,payload):
+    headers = {
+        "x-dassana-app-id":f"{app_id}",
+        "x-dassana-token":f"{dassana_token}",
+        "x-dassana-ingestion-type":f"{ingestion_type}",
+        "x-dassana-snapshot-id":f"{snapshot_id}"
+    }
+    resp = requests.put(f"https://ingestion.dassana.dev/snapshot", headers=headers,json=payload)
+    updated_snapshot = resp.json()
+    return updated_snapshot
+
+    
+def end_snapshot(ingestion_type,snapshot_id,status,is_recon):
+    headers = {
+        "x-dassana-app-id":f"{app_id}",
+        "x-dassana-token":f"{dassana_token}",
+        "x-dassana-ingestion-type":f"{ingestion_type}",
+        "x-dassana-snapshot-id":f"{snapshot_id}"
+    }
+    resp = requests.post(f"https://ingestion.dassana.dev/snapshot/{status}?isReconJob={is_recon}", headers=headers)
+    return resp.status_code
+
+def forward_logs(log_data,type="findings",snapshot_id=None):    
     if type == 'findings':
         endpoint = f"{os.environ['DASSANA_ENDPOINT']}/findings"
     elif type == 'assets':
@@ -189,11 +315,14 @@ def forward_logs(log_data,type="findings"):
     magic_word = get_magic_word()
 
     headers = {
-        "x-dassana-token": token,
-        "x-dassana-app-id": app_id,
-        "Content-type": "application/x-ndjson",
-        "Content-encoding": "gzip",
-    }
+            "x-dassana-token": token,
+            "x-dassana-app-id": app_id,
+            "Content-type": "application/x-ndjson",
+            "Content-encoding": "gzip",
+        }
+    
+    if snapshot_id:
+        headers['x-dassana-snapshot-id'] = snapshot_id
 
     if magic_word:
         headers['x-dassana-magic-word'] = magic_word
@@ -268,3 +397,39 @@ def acknowledge_delivery():
     subscriber.acknowledge(
         request={"subscription": subscription_path, "ack_ids": ack_ids}
     )
+
+def refreshDassanaToken(token):
+    decoded_token = jwt.decode(token, key=None, options={"verify_signature":False})
+
+    # Less than 3 minutes to expiry, refresh
+    if decoded_token.get('exp') - int(time.time()) < 180:
+        return get_access_token()
+    return token
+
+def post_dassana_query(query: str, timeRange: TimeRange) -> list:
+    try:
+        token = get_access_token()
+        #refresh
+        token = refreshDassanaToken(token)
+        decoded = jwt.decode(token, key=None, options={"verify_signature":False})
+        domain = (str(decoded["aud"]).split(".")[2])
+
+        requestHeader = {
+            "content-type" : "application/json",
+            "Authorization" : "Bearer"+" "+token,
+            "Accept" : "application/json",
+            "x-dassana-tenant-id" : tenant_id
+        }
+
+        requestBody = dict()
+        requestBody["query"] = query
+        requestBody["timeRange"] = timeRange.getTimeRangeDict()
+
+        response = requests.request("POST", f"https://dquery.dassana.{domain}/query", headers=requestHeader, json=requestBody).text
+        responseDict = json.loads(response)
+        return responseDict
+
+    except KeyError as e:
+        logging.error(f"No such key '{e}' found while triggering dassana query")
+    except Exception as e:
+        logging.error(f"An exception occurred while triggering dassana query {e}")
